'---------------------------------------------------------------------------------------------------------
'Memory Access Functions

' Read functions:
' address - address in sram
' data - return value

read_sram_byte
        call #ram_address               ' sets up the latches with the correct ram address
        call #read_memory_byte
        mov data, data_8
read_sram_byte_ret
        ret

read_sram_word
        call #ram_address               ' sets up the latches with the correct ram address
        call #read_memory_byte
        mov data, data_8
        shl data, #8
        call #ram_address_next
        call #read_memory_byte
        or data, data_8
        sub address, #1
read_sram_word_ret
        ret

read_sram_long
        call #ram_address               ' sets up the latches with the correct ram address
        call #read_memory_byte
        mov data, data_8
        shl data, #8
        call #ram_address_next
        call #read_memory_byte
        or data, data_8
        shl data, #8
        call #ram_address_next
        call #read_memory_byte
        or data, data_8
        shl data, #8
        call #ram_address_next
        call #read_memory_byte
        or data, data_8
        sub address, #3
read_sram_long_ret
        ret

' Write functions:
' address - address in sram
' data - data to write

write_sram_byte
        call #ram_address               ' sets up the latches with the correct ram address
        mov data_8, data
        call #write_memory_byte
write_sram_byte_ret
        ret

write_sram_word
        call #ram_address               ' sets up the latches with the correct ram address
        mov data_8, data
        rol data_8, #24
        call #write_memory_byte
        call #ram_address_next
        rol data_8, #8
        call #write_memory_byte
        sub address, #1
write_sram_word_ret
        ret

write_sram_long
        call #ram_address               ' sets up the latches with the correct ram address
        mov data_8, data
        rol data_8, #8
        call #write_memory_byte
        call #ram_address_next
        rol data_8, #8
        call #write_memory_byte
        call #ram_address_next
        rol data_8, #8
        call #write_memory_byte
        call #ram_address_next
        rol data_8, #8
        call #write_memory_byte
        sub address, #3
write_sram_long_ret
        ret

''From Dracblade driver for talking to a ram chip via three latches
'' Modified code from Cluso's triblade

read_memory_byte
        mov dira,LatchDirectionRd       ' for reads so P0-P7 tristate till do read
        mov outa,ReadEnable             ' or with correct 138 address
        andn outa,GateHigh              ' set gate low
        nop                             ' short delay to stabilise
        nop
        mov data_8, ina                 ' read SRAM
        and data_8, #$FF                ' extract 8 bits
        or  outa,GateHigh               ' set the gate high again
read_memory_byte_ret
        ret

write_memory_byte
        mov outx,data_8                 ' get the byte to output
        and outx, #$FF                  ' ensure upper bytes=0
        or outx,WriteEnable             ' or with correct 138 address
        mov outa,outx                   ' send it out
        andn outa,GateHigh              ' set gate low
        nop                             ' no nop doesn't work, one does, so put in two to be sure
        nop                             ' another NOP
        or outa,GateHigh                ' set it high again
write_memory_byte_ret
        ret

ram_address

        mov dira,LatchDirectionWr       ' set up the pins for programming latch chips

        ' set bits 23:16
        mov outx,address                ' get the address into a temp variable
        shr outx,#16                    ' bits 23:16
        and outx,#$FF                   ' mask the low byte
        or  outx,HighAddress            ' or with 138 low address
        mov outa,outx                   ' send it out
        andn outa,GateHigh              ' set gate low
        or outa,GateHigh                ' set it high again
                        
        ' set bits 15:8
        mov outx,address                ' get the address into a temp variable
        shr outx,#8                     ' bits 15:8
        and outx,#$FF                   ' mask the low byte
        or  outx,MiddleAddress          ' or with 138 middle address
        mov outa,outx                   ' send it out
        andn outa,GateHigh              ' set gate low
        or outa,GateHigh                ' set it high again
        jmp #ram_address_low
                        
ram_address_next

        add address,#1
        mov dira,LatchDirectionWr       ' set up the pins for programming latch chips

ram_address_low

        ' set bits 7:0
        mov outx,address                ' get the address into a temp variable
        and outx,#$FF                   ' mask the low byte
        or  outx,LowAddress             ' or with 138 middle address
        mov outa,outx                   ' send it out
        andn outa,GateHigh              ' set gate low
        or outa,GateHigh                ' set it high again
        
ram_address_ret
ram_address_next_ret
        ret

LatchMask               ' same as LatchDirection below               ' mask of all bits used by the SDRAM interface
LatchDirectionWr        long    %00000000_00000000_00001111_11111111 ' 138 active, gate active and 8 data lines active
LatchDirectionRd        long    %00000000_00000000_00001111_00000000 ' for reads so data lines are tristate till the read
GateHigh                ' same as ReadEnable below                   ' HC138 gate high, all others must be low
ReadEnable              long    %00000000_00000000_00000001_00000000 ' xxxx000x = /RD and gate high xxxxxxx1
WriteEnable             long    %00000000_00000000_00000011_00000000 ' xxxx001x = /WE and gate high xxxxxxx1
LowAddress              long    %00000000_00000000_00000101_00000000 ' xxxx010x = low address latch and gate high xxxxxxx1
MiddleAddress           long    %00000000_00000000_00000111_00000000 ' xxxx011x = middle address latch and gate high xxxxxxx1
HighAddress             long    %00000000_00000000_00001001_00000000 ' xxxx100x = high address latch and gate high xxxxxxx1

data_8                  long    0                                    ' data to read from or write to memory
outx                    long    0                                    ' for temp use
